@@ -1,6 +1,7 @@
 import csv
 import numpy as np
 from six import string_types
+from inspect import signature
 
 from bmtk.simulator.core.simulator import Simulator
 import bmtk.simulator.utils.simulation_inputs as inputs
@@ -9,6 +10,9 @@ from bmtk.simulator.filternet.lgnmodel.movie import *
 from bmtk.simulator.filternet import modules as mods
 from bmtk.simulator.filternet.io_tools import io
 from bmtk.utils.io.ioutils import bmtk_world_comm
+from bmtk.simulator.filternet.auditory_processing import AuditoryInput
+import scipy.io as syio
+import os
 
 
 class FilterSimulator(Simulator):
@@ -72,8 +76,10 @@ class FilterSimulator(Simulator):
             self._movies.append(mv)
 
         elif movie_type == 'graiting':
-            init_params = FilterSimulator.find_params(['row_size', 'col_size', 'frame_rate'], **params)
-            create_params = FilterSimulator.find_params(['gray_screen_dur', 'cpd', 'temporal_f', 'theta', 'contrast'],
+            init_param_names = list(signature(GratingMovie.__init__).parameters.keys())
+            create_param_names = list(signature(GratingMovie.create_movie).parameters.keys())
+            init_params = FilterSimulator.find_params(init_param_names, **params)
+            create_params = FilterSimulator.find_params(create_param_names,
                                                         **params)
 
             create_params['gray_screen_dur'] /= 1000.0
@@ -96,6 +102,74 @@ class FilterSimulator(Simulator):
         else:
             self._eval_options.append({})
 
+    def add_audio(self, audio_type, params):
+        # Create cochleagram "movie" from audio wav file
+        audio_type = audio_type.lower() if isinstance(audio_type, string_types) else 'movie'
+        if audio_type in ['wav_file', 'mat_file'] or not audio_type:
+            if 'data_file' in params:
+                aud_file = params['data_file']
+                if audio_type == 'mat_file':
+                    n = params['stim_number']
+                    wav_file = os.path.splitext(aud_file)[0] + str(n) + '.wav'
+                    if not os.path.exists(wav_file):
+                        mat = syio.loadmat(params['data_file'])
+                        data = np.squeeze(mat['timit_sents'][0, n])
+                        sr = mat['aud_fs'][0][0]
+                        scaled = np.int16(data / np.max(np.abs(data)) * 32768)
+                        syio.wavfile.write(wav_file, sr, scaled)
+                    else:
+                        io.log_warning('Wav file already exists, please delete to overwrite.')
+                    aud_file = wav_file
+
+                #elif 'data' in params:
+                #    m_data = params['data']
+            else:
+                raise Exception('Could not find audio "data_file" in config to use as input.')
+
+            aud = AuditoryInput(aud_file)
+
+            #if params.get('frame_rate'):
+            #    frame_rate = params.get('frame_rate')
+            #else:
+            init_params = FilterSimulator.find_params(['row_range', 'col_range', 'labels', 'units', 'frame_rate',
+                                                       't_range', 'padding'], **params)
+            if 'frame_rate' in init_params.keys():
+                frame_rate = init_params['frame_rate']
+            else:
+                frame_rate = 1000
+
+            coch, center_freqs_log, times = aud.get_cochleagram(frame_rate, interp_to_freq=params['interp_to_freq'])
+            coch = coch.T
+            #coch = np.log(coch)
+
+            normalize_data = params.get('normalize', None)
+            if normalize_data == 'full' or normalize_data == True:
+                contrast_min, contrast_max = coch.min(), coch.max()
+                self.io.log_info('Normalizing auditory input to (-1.0, 1.0).')
+                coch = (coch-contrast_min)*2.0/(contrast_max - contrast_min) - 1.0
+            elif normalize_data == 'relative':
+                self.io.log_info('Auditory input is normalized maintaining relative amplitude')
+                coch = coch*2.8
+            else:
+                self.io.log_info('Auditory input range is not normalized.')
+
+            coch = coch[:,:, np.newaxis]
+
+            # Note, overwrites these if user supplied, instead taken from cochleagram
+            init_params['row_range'] = center_freqs_log
+            init_params['col_range'] = [0]
+            init_params['t_range'] = times
+            #? Frame_rate
+            # Dimensions of time, row, column
+            self._movies.append(Movie(coch, **init_params))
+        else:
+            raise Exception('Unknown audio type {}'.format(audio_type))
+
+        if 'evaluation_options' in params:
+            self._eval_options.append(params['evaluation_options'])
+        else:
+            self._eval_options.append({})
+
     def run(self):
         for mod in self._sim_mods:
             mod.initialize(self)
@@ -107,15 +181,21 @@ class FilterSimulator(Simulator):
         ten_percent = int(np.ceil(n_cells_on_rank*0.1))
         rank_msg = '' if bmtk_world_comm.MPI_size < 2 else ' (on rank {})'.format(bmtk_world_comm.MPI_rank)
 
+        max_fr = np.empty(len(cells_on_rank))
         for cell_num, cell in enumerate(cells_on_rank):
             for movie, options in zip(self._movies, self._eval_options):
                 if cell_num > 0 and cell_num % ten_percent == 0:
                     io.log_debug(' Processing cell {} of {}{}.'.format(cell_num, n_cells_on_rank, rank_msg))
                 ts, f_rates = cell.lgn_cell_obj.evaluate(movie, **options)
+                max_fr[cell_num] = np.max(f_rates)
+                if movie.padding:
+                    f_rates = f_rates[int(movie.data.shape[0]-movie.data_orig.shape[0]):]
+                    ts = ts[int(movie.data.shape[0]-movie.data_orig.shape[0]):]
+                    ts = ts-ts[0]
 
                 for mod in self._sim_mods:
                     mod.save(self, cell, ts, f_rates)
-
+        io.log_info('Max firing rate: {}'.format(np.max(max_fr)))
         io.log_info('Done.')
         for mod in self._sim_mods:
             mod.finalize(self)
@@ -151,6 +231,8 @@ class FilterSimulator(Simulator):
         for sim_input in inputs.from_config(config):
             if sim_input.input_type == 'movie':
                 sim.add_movie(sim_input.module, sim_input.params)
+            elif sim_input.input_type == 'audio':
+                sim.add_audio(sim_input.module, sim_input.params)
             else:
                 raise Exception('Unable to load input type {}'.format(sim_input.input_type))
 
